\name{multiget}
\alias{multiget}
\title{Return a list containing the Values of the supplied variables.}
\description{
  Search for an \R objects with a given names and return the values found.
}
 
\usage{
multiget(x, pos=-1, envir=as.environment(pos), mode="any",
inherits=TRUE, iffail)
}
\arguments{
  \item{x}{a vector or list of variable names (given as a quoted
    character strings).} 
  \item{pos}{where to look for the object (see the details section); if
    omitted, the function will search, as if the name of the object
    appeared in unquoted in an expression.
  }
  \item{envir}{an alternative way to specify an environment to look in;
    see the details section.}
  \item{mode}{the mode of object sought.}
  \item{inherits}{should the enclosing frames of the environment be
    inspected?}
  \item{iffail}{A value to be used for items in the list that are not found.}
}
\details{
  The \code{pos} argument can specify the  environment in which to look
  for the object in any of several ways:
  as an integer (the position in the \code{\link{search}} list); as
  the character string name of an element in the search list; or as an
  \code{\link{environment}} (including using \code{\link{sys.frame}} to
  access the currently active function calls).
  The \code{envir} argument is an alternative way to specify an
  environment, but is primarily there for back compatibility.
  
   The \code{mode} includes collections such as \code{"numeric"} and
   \code{"function"}: any member of the collection will suffice. 
}
\value{
  A named list (the names come from \code{x}) of the variables found.
  If \code{iffail} was specified then any object named in \code{x} that
  is not found will have the value of \code{iffail} as its value.
  Each variable lookup is wrappted in a \code{try} call so that failure
  to find one variable does not prevent finding others.
  
  This function searches the specified environment for a bound variable
  whose name is given by the character string \code{x}.  If the
  variable's value is not of the correct \code{mode}, it is ignored.

  If \code{inherits} is \code{FALSE}, only the first frame of the
  specified environment is inspected.  If \code{inherits} is
  \code{TRUE}, the search is continued up through the parent frames
  until a bound value of the right mode is found.

  Using a \code{NULL} environment is equivalent to using the current
  environment.
}
\seealso{
  \code{\link{exists}}, \code{\link{multiassign}}.
}
\examples{
nam <- paste("r",1:6, sep=".")

multiassign(nam, 11:16)
rm("r.3")
multiget(nam, iffail=NA)

}
\keyword{data}
